/*
    Copyright (C) 2012 NewestTools.com(AlexLe)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "ExplorerContextMenu.h"
#include "DbgPrintf.h"
#include "WinTools.h"
#include "PasteFilesFromHost.h"
#include "LangTools.h"

//TODO: .lnk files support (K:\Users\w7sp1\Desktop)

STDMETHODIMP_(ULONG) 
CExplorerContextMenu::AddRef()
{
    return ++m_cRef;
}

STDMETHODIMP_(ULONG) 
CExplorerContextMenu::Release()
{
    ULONG cRef = --m_cRef;

    if ( cRef == 0 )
        delete this;

    return cRef;
}

STDMETHODIMP
CExplorerContextMenu::QueryInterface(
    REFIID riid,
    PVOID *ppv
    )
{
    *ppv = NULL;

    if ( IsEqualIID( riid, IID_IUnknown ) )
    {
        DbgPrintf( "IID_IUnknown referenced\n" );
        *ppv = (this);
    }

    if ( IsEqualIID( riid, IID_IShellExtInit ) )
    {
        DbgPrintf( "IID_IShellExtInit referenced\n" );
        *ppv = ((IShellExtInit *)this);
    }
         

    if ( IsEqualIID( riid, IID_IContextMenu ) )
    {
        DbgPrintf( "IID_IContextMenu referenced\n" );
        *ppv = ((IContextMenu *)this);
    }

    if ( *ppv != NULL )
    {
        AddRef();
        return NOERROR;
    }

    return E_NOINTERFACE;
    
}

BOOL
IsDirectoryW( LPCWSTR lpFileName )
{
    DWORD dwFileAttr = GetFileAttributesW( lpFileName );

    if ( dwFileAttr != -1 && ( dwFileAttr & FILE_ATTRIBUTE_DIRECTORY ) )
    {
        return TRUE;
    }

    return FALSE;

}

STDMETHODIMP
CExplorerContextMenu::Initialize(
    LPCITEMIDLIST pIDFolder,
    LPDATAOBJECT pDataObj,
    HKEY hKeyID
    )
{
    FORMATETC Format = { CF_HDROP, NULL, DVASPECT_CONTENT, -1, TYMED_HGLOBAL };
    STGMEDIUM stgMedium = { TYMED_HGLOBAL, NULL, NULL };
    BOOL bRet = FALSE;

    //DbgPrintf( "CExplorerContextMenu->Initialize called pIDFolder=%p\n", pIDFolder );

    m_Folder = L"";

    if ( pDataObj == NULL )
    {
        //Background folders support 

        DbgPrintf( "pDataObj == NULL\n" );

        WCHAR wszPath[ MAX_PATH + 1 ] = L"";

        bRet = SHGetPathFromIDListW( pIDFolder, wszPath );

        DbgPrintf( "SHGetPathFromIDListW return %u, %ws\n", (ULONG)bRet, wszPath );

        if ( bRet && IsDirectoryW(wszPath) )
        {
            m_Folder = wszPath;
        }

        return NOERROR;
    }

    HRESULT hr = pDataObj->GetData(
        &Format,
        &stgMedium
        );

    if ( !SUCCEEDED( hr ) || NULL == stgMedium.hGlobal )
    {
        DbgPrintf( "pDataObj->GetData failed\n" );
        return NOERROR;
    }

    UINT FileCount = DragQueryFile((HDROP)stgMedium.hGlobal, (UINT)-1, 0, 0);

    //handle only 1 folder

    if ( FileCount != 1 )
    {
        DbgPrintf( "FileCount is not accepted(%u)\n", FileCount );
        return NOERROR;
    }

    WCHAR wszFileName[ MAX_PATH ] = L"";

    int ret = DragQueryFileW(
        (HDROP)stgMedium.hGlobal,
        0,
        wszFileName,
        ARRAYSIZE(wszFileName)
        );

    if ( ret <= 0 )
    {
        DbgPrintf( "DragQueryFileW failed\n" );
        return NOERROR;
    }

    DbgPrintf( "FileName: %ws\n", wszFileName );

    DWORD dwFileAttr = GetFileAttributesW( wszFileName );

    if ( dwFileAttr != -1 && ( dwFileAttr & FILE_ATTRIBUTE_DIRECTORY ) )
    {
        m_Folder = wszFileName;
    }
    else
    {
        if ( IsEqualFileExtW(wszFileName, L".lnk" ) )
        {
            wstring strLinkPath = COM_LnkToFilePathW( wszFileName );

            if ( strLinkPath != L"" && 
                 (dwFileAttr = GetFileAttributesW( strLinkPath.c_str() )) != -1 &&
                ( dwFileAttr & FILE_ATTRIBUTE_DIRECTORY )
               )
            {
                m_Folder = strLinkPath;
                DbgPrintf( "LNK point to directory: %ws\n", strLinkPath.c_str() );
            }
        }
    }

    return NOERROR;

}


STDMETHODIMP
CExplorerContextMenu::QueryContextMenu(
    HMENU hShellMenu,
    UINT indexMenu,
    UINT idCmdFirst,
    UINT idCmdLast,
    UINT uFlags
    )
{
    UINT MenuItemCount = 0;

    //DbgPrintf( "CExplorerContextMenu::QueryContextMenu called\n" );

    //TODO: LangLoadString

    if ( m_Folder.length() != 0 )
    {
        BOOL bRet = InsertMenuW(
            hShellMenu,
            indexMenu,
            MF_BYPOSITION | MF_STRING,
            idCmdFirst,
            LangGetStringW( IDS_STR_GUEST_OS_PASTE_FILES ).c_str()
            );

        if ( bRet )
		{
            MenuItemCount++;
						
			HBITMAP hMenuBmp ;
									
			hMenuBmp = (HBITMAP)LoadImage( 
				g_DllHinst, 
				MAKEINTRESOURCE(IDI_APP_BMP),
				IMAGE_BITMAP,
				GetSystemMetrics(SM_CXMENUCHECK),
				GetSystemMetrics(SM_CYMENUCHECK),
				LR_CREATEDIBSECTION | LR_SHARED
				);
						
			if ( hMenuBmp )			
			{
				SetMenuItemBitmaps(
					hShellMenu,
					indexMenu,
					MF_BYPOSITION,
					hMenuBmp,
					hMenuBmp					
					);					
			}					
		}    
	}

    return MenuItemCount;
}


STDMETHODIMP
CExplorerContextMenu::InvokeCommand( LPCMINVOKECOMMANDINFO lpcmi )
{
  HRESULT hr = S_OK;

  //DbgPrintf( "CExplorerContextMenu::InvokeCommand called\n" );

  switch ( (UINT_PTR)(lpcmi->lpVerb) )
  {
    case 0:
    {
	    //MessageBoxW( NULL, m_Folder.c_str(), L"Info", MB_SETFOREGROUND );

        if ( m_Folder != L"" )
        {
            PasteFilesFromHost( m_Folder.c_str() );
        }

        break;
    }
  }
  return hr;
}

